@@ -32,12 +32,14 @@
 #include <gconf/gconf-client.h>
 #include <nautilus-burn-drive.h>
 #include <libgnome/gnome-help.h>
+#include <libgnome/gnome-url.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <profiles/gnome-media-profiles.h>
 #include <gst/gst.h>
 
 #include "bacon-message-connection.h"
+#include "gconf-bridge.h"
 #include "sj-about.h"
 #include "sj-metadata.h"
 #include "sj-metadata-musicbrainz.h"
@@ -70,6 +72,7 @@ static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_entry;
 static GtkWidget *track_listview, *extract_button, *play_button;
 static GtkWidget *status_bar;
 static GtkWidget *extract_menuitem, *play_menuitem, *next_menuitem, *prev_menuitem, *select_all_menuitem, *deselect_all_menuitem;
+static GtkWidget *submit_menuitem;
 GtkListStore *track_store;
 static BaconMessageConnection *connection;
 GtkCellRenderer *toggle_renderer, *title_renderer, *artist_renderer;
@@ -83,9 +86,13 @@ gboolean extracting = FALSE;
 static gint total_no_of_tracks;
 static gint no_of_tracks_selected;
 static AlbumDetails *current_album;
+static char *current_submit_url = NULL;
 
 gboolean autostart = FALSE, autoplay = FALSE;
 
+/* For the default drive stuff, TODO: remove */
+static GList *cdroms = NULL;
+
 #define DEFAULT_PARANOIA 4
 #define RAISE_WINDOW "raise-window"
 #define SOURCE_GLADE "../data/sound-juicer.glade"
@@ -652,6 +659,10 @@ static void reread_cd (gboolean ignore_no_media)
 
   d(if (!drive) g_printerr("Attempting to re-read NULL drive\n"));
 
+  g_free (current_submit_url);
+  current_submit_url = NULL;
+  gtk_widget_set_sensitive (submit_menuitem, FALSE);
+
   if (!is_audio_cd (drive)) {
     d(g_printerr("Media is not an audio CD\n"));
     update_ui_for_album (NULL);
@@ -660,7 +671,12 @@ static void reread_cd (gboolean ignore_no_media)
       gdk_window_set_cursor (main_window->window, NULL);
     return;
   }
-  
+
+  current_submit_url = sj_metadata_get_submit_url (metadata);
+  if (current_submit_url) {
+    gtk_widget_set_sensitive (submit_menuitem, TRUE);
+  }
+
   sj_metadata_list_albums (metadata, &error);
 
   if (error && !(error->code == SJ_ERROR_CD_NO_MEDIA && ignore_no_media)) {
@@ -775,12 +791,41 @@ set_device (const char* device, gboolean ignore_no_media)
   sj_metadata_set_cdrom (metadata, device);
   sj_extractor_set_device (extractor, device);
   
-  tray_opened = nautilus_burn_drive_door_is_open (drive);
-  if (tray_opened == FALSE) {
-    reread_cd (ignore_no_media);
+  if (drive != NULL) {
+    tray_opened = nautilus_burn_drive_door_is_open (drive);
+    if (tray_opened == FALSE) {
+      reread_cd (ignore_no_media);
+    }
   }
 }
 
+gboolean cd_drive_exists (const char *device)
+{
+  GList *l;
+  if (device == NULL) return FALSE;
+  if (cdroms == NULL) {
+    cdroms = nautilus_burn_drive_get_list (FALSE, FALSE);
+  }
+  for (l = cdroms; l != NULL; l = l->next) {
+    if (strcmp (((NautilusBurnDrive *) (l->data))->device, device) == 0)
+      return TRUE;
+  }
+  return FALSE;
+}
+
+const char* prefs_get_default_device ()
+{
+  static const char* default_device = NULL;
+  if (default_device == NULL) {
+    NautilusBurnDrive *cd;
+    cdroms = nautilus_burn_drive_get_list (FALSE, FALSE);
+    if (cdroms == NULL) return NULL;
+    cd = cdroms->data;
+    default_device = cd->device;
+  }
+  return default_device;
+}
+
 /**
  * The GConf key for the device changed
  */
@@ -904,6 +949,35 @@ void on_reread_activate (GtkWidget *button, gpointer user_data)
   reread_cd (FALSE);
 }
 
+/**
+ * Clicked the Submit menu item in the UI
+ */
+void on_submit_activate (GtkWidget *menuitem, gpointer user_data)
+{
+  GError *error = NULL;
+
+  if (current_submit_url) {
+    if (!gnome_url_show (current_submit_url, &error)) {
+      GtkWidget *dialog;
+
+      dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
+                                       GTK_DIALOG_DESTROY_WITH_PARENT,
+                                       GTK_MESSAGE_ERROR,
+                                       GTK_BUTTONS_CLOSE,
+                                       "<b>%s</b>\n\n%s\n%s: %s",
+				       _("Could not open URL"),
+				       _("Sound Juicer could not open the submission URL"),
+				       _("Reason"),
+				       error->message);
+      gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+      g_error_free (error);
+    }
+  }
+
+}
+
 /**
  * Called in on_extract_toggled to see if there are selected tracks or not.
  * extracting points to the boolean to set to if there are tracks to extract,
@@ -1142,29 +1216,35 @@ int main (int argc, char **argv)
   GError *error = NULL;
   GtkTreeSelection *selection;
   char *device = NULL;
-  struct poptOption options[] = {
-    { NULL, '\0', POPT_ARG_INCLUDE_TABLE, NULL, 0, "GStreamer", NULL },
-    { "auto-start", 'a', POPT_ARG_NONE, &autostart, 0, "Start extracting immediately", NULL},
-    { "play", 'p', POPT_ARG_NONE, &autoplay, 0, "Start playing immediately", NULL},
-    { "device", 'd', POPT_ARG_STRING, &device, 0, "What CD device to read", NULL},
-    POPT_TABLEEND
+  GOptionContext *ctx;
+  const GOptionEntry entries[] = {
+    { "auto-start", 'a', 0, G_OPTION_ARG_NONE, &autostart, N_("Start extracting immediately"), NULL },
+    { "play", 'p', 0, G_OPTION_ARG_NONE, &autoplay, N_("Start playing immediately"), NULL},
+    { "device", 'd', 0, G_OPTION_ARG_FILENAME, &device, N_("What CD device to read"), N_("DEVICE") },
+    { NULL }
   };
 
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
   
-  options[0].arg = (void *) gst_init_get_popt_table ();
-  gnome_program_init ("sound-juicer",
-                      VERSION, LIBGNOMEUI_MODULE,
-                      argc, argv,
-                      GNOME_PROGRAM_STANDARD_PROPERTIES,
-                      GNOME_PARAM_POPT_TABLE, options,
-                      NULL);
   g_set_application_name (_("Sound Juicer"));
+  
+  ctx = g_option_context_new ("- Sound Juicer");
+  g_option_context_add_main_entries (ctx, entries, GETTEXT_PACKAGE);  
+  g_option_context_add_group (ctx, gtk_get_option_group (FALSE));
+  g_option_context_add_group (ctx, gst_init_get_option_group ());
+  if (!g_option_context_parse (ctx, &argc, &argv, &error)) {
+    g_critical ("Failed to initialize: %s\n", error->message);
+    g_error_free (error);
+    return 1;
+  }
 
+  gtk_init (&argc, &argv);
   sj_stock_init ();
 
+  gtk_window_set_default_icon_from_file (PIXMAPDIR"/sound-juicer.png", NULL);
+
   connection = bacon_message_connection_new ("sound-juicer");
   if (bacon_message_connection_get_is_server (connection) == FALSE) {
 	  bacon_message_connection_send (connection, RAISE_WINDOW);
@@ -1222,10 +1302,9 @@ int main (int argc, char **argv)
   glade_xml_signal_autoconnect (glade);
 
   main_window = glade_xml_get_widget (glade, "main_window");
-  gtk_window_set_icon_from_file (GTK_WINDOW (main_window), PIXMAPDIR"/sound-juicer.png", NULL);
-
   select_all_menuitem = glade_xml_get_widget (glade, "select_all");
   deselect_all_menuitem = glade_xml_get_widget (glade, "deselect_all");
+  submit_menuitem = glade_xml_get_widget (glade, "submit");
   title_entry = glade_xml_get_widget (glade, "title_entry");
   artist_entry = glade_xml_get_widget (glade, "artist_entry");
   duration_label = glade_xml_get_widget (glade, "duration_label");
@@ -1307,9 +1386,6 @@ int main (int argc, char **argv)
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
   }
 
-  update_ui_for_album (NULL);
-  gtk_widget_show (main_window);
-
   extractor = SJ_EXTRACTOR (sj_extractor_new());
   error = sj_extractor_get_new_error (extractor);
   if (error) {
@@ -1317,11 +1393,15 @@ int main (int argc, char **argv)
     exit (1);
   }
 
+  update_ui_for_album (NULL);
+  gconf_bridge_bind_window_size(gconf_bridge_get(), GCONF_WINDOW, GTK_WINDOW (main_window));
+  gtk_widget_show (main_window);
+
   sj_play_init ();
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (track_listview));
   gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
-	  
+
   http_proxy_setup (gconf_client);
   baseuri_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_BASEURI, NULL, TRUE, NULL), NULL);
   path_pattern_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_PATH_PATTERN, NULL, TRUE, NULL), NULL);
